﻿local addonName, addon = ...
local L = addon

SC_MAX_SPELLS = 30
SC_DELTA = 2
SC_PREMSG = "|cFF00CC00SC:|r "
addon.events = CreateFrame("Frame")
addon.sv = {}

addon.sv.defaults_global = {
	["Pos"] = {
		["X"] = 500,
		["Y"] = 400,
	},
	["Movable"] = true,
	["Scale"] = 1,
	["Reset"] = true,
	["Friendly"] = false,
	["onlyFriendly"] = false,
	["Timer"] = false,
	["Cooldown"] = false,
	["Size"] = {
		["Width"] = 318,
		["Height"] = 106,
	},
	["Version"] = "1.2.3",
}

function addon:InsertVariables(varsSrc, varsDst, output)
	for key, value in pairs(varsSrc) do
		if varsDst[key] == nil then
			varsDst[key] = value
			if output then print(SC_PREMSG.."variable inserted: "..key) end
		elseif type(value) == "table" then
			varsDst[key] = self:InsertVariables(value, varsDst[key], output)
		end
	end
	return varsDst
end

function addon:RemoveVariables(varsSrc, varsDst, output)
	for key, value in pairs(varsSrc) do
		if varsDst[key] == nil then
			varsSrc[key] = nil
			if output then print(SC_PREMSG.."variable removed: "..key) end
		elseif type(value) == "table" then
			varsDst[key] = self:RemoveVariables(value, varsDst[key], output)
		end
	end
	return varsDst
end

function addon:UpdateVariables()
	if self.sv.global.Version == self.version and self.sv.global.Version ~= nil then return end
	self.sv.global = self:InsertVariables(self.sv.defaults_global, self.sv.global)
	self.sv.defaults_global = self:RemoveVariables(self.sv.global, self.sv.defaults_global)
	self.sv.global.Version = self.version
	print(SC_PREMSG..L["The addon was successfully updated!"])
end

function SCMainFrame_OnLoad(self)
	self.currentID = 1
	self.chargedSpells = {}
	self.wotf = {}
	self.header:SetText(L["Left click - move   Right click - configuration"])
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:SetScript("OnEvent", SCMainFrame_OnEvent)
end

function SCMainFrame_CorrectSize(self)
	w, h = self:GetSize() print(w,h)
	h = floor(h) print(h)
	if mod(h,2) == 1 then h = h - 1 end print(h)
	w = 3 * h
	print(w,h)
	self:SetSize(w,h)
end

function SCMainFrame_UpdateFramesSize(self)
	w, h = self:GetSize()
	w = floor((w - 18) / 10)
	h = w
	print(w,h)
	for i = 1, SC_MAX_SPELLS do
		self[i]:SetSize(w,h)
	end
end

function SCMainFrame_UpdateFramesPositions(self, id)
	local endID
	local scale = addon.sv.global.Scale
	if not id then
		endID = SC_MAX_SPELLS
		id = 1
	else
		endID = self.currentID
		local temp = self[id]
		for i = id, endID - 2 do
			self[i] = self[i+1]
			self[i]:SetID(self[i]:GetID() - 1)
			if	self.chargedSpells[self[i].unitGUID] then
				if self.chargedSpells[self[i].unitGUID][self[i].spell] then
					self.chargedSpells[self[i].unitGUID][self[i].spell] = self[i]:GetID()
				end
			end
		end
		self.currentID = self.currentID - 1
		temp:SetID(self.currentID)
		self[self.currentID] = temp
		endID = endID - 1
	end
	local w,h = self[1]:GetSize()
	for i = id, endID do
		self[i]:SetPoint("TOPLEFT", (w + SC_DELTA) * mod(i-1,10), -12 - (h + SC_DELTA) * floor((i-1)/10))
	end
end
--/run SCMainFrame_UpdateFramesPositions(SCMainFrame)

local function FilterEvent(e, spell, unit, wotf)
	if e.time and GetTime()-e.time <0.05 and e.spell==spell and UnitGUID(unit)==UnitGUID(e.unit) then
		e.time = GetTime()
		return false
	end
	if not wotf then
		e.time = GetTime()
		e.spell = spell
		e.unit = unit
	end
	return true
end

function SCSpellFrame_SetTimer(self, hasTimer)
	if hasTimer then
		local time = self.dur - (GetTime() - self.start)
		if time <= 5 then
			self.timer:SetText(format("%1.1f", time))
			self.timer:SetTextColor(1,0,0,1)
		elseif time < 10 then
			self.timer:SetText(format("%1.1f", time))
			self.timer:SetTextColor(1,1,0,1)
		elseif time < 60 then
			self.timer:SetText(floor(time))
			self.timer:SetTextColor(1,1,0,1)
		else
			local sec = floor(mod(time,60))
			if sec < 10 then sec = "0"..sec end
			time =  floor(time/60)..":"..sec
			self.timer:SetText(time)
			self.timer:SetTextColor(1,1,1,1)
		end
	end
end

function SCSpellFrame_ResetCooldown(self)
	self.texture:SetTexture(0,0,1,0.6)
	self.cd:SetCooldown(GetTime(), 0)
	self.spell = nil	
	self.unitGUID = nil
	self.dur = nil
	self.start = nil
	self.charges:SetText("")
	self.ch = nil
	SCMainFrame_UpdateFramesPositions(SCMainFrame, self:GetID())
	if not addon.sv.global.Movable then self:Hide() end
	self:SetScript("OnUpdate", nil)
end

function SCMainFrame_ResetCooldowns(self)
	while self.currentID > 1 do
		SCSpellFrame_ResetCooldown(self[self.currentID-1])
	end
	self.chargedSpells = {}
	self.wotf = {}
end

function SCMainFrame_OnPrepUsed(self, unitGUID)
	local i = 1
	repeat
	if self.currentID == 2 then break end
		if self[i].unitGUID == unitGUID and addon.resets[self[i].spell] then
			--print(i," ",self[i].spell)
			SCSpellFrame_ResetCooldown(self[i])
		else
			i = i + 1
		end
	until i == self.currentID - 1
end

function SCMainFrame_SetToMove(self, move)
	self:SetMovable(move)
	if move then
		self.texture:Show()
		self.header:Show()
		self:SetScript("OnMouseDown", SCMainFrame_OnMouseDown)
		self:SetScript("OnMouseUp", SCMainFrame_OnMouseUp)
		for i = self.currentID, SC_MAX_SPELLS do
			self[i]:Show()
		end
	else
		self.texture:Hide()
		self.header:Hide()
		self:SetScript("OnMouseDown", nil)
		self:SetScript("OnMouseUp", nil)
		for i = self.currentID, SC_MAX_SPELLS do
			self[i]:Hide()
		end
	end
end

function SCMainFrame_OnSizeChanged(self, w, h)
	 SCMainFrame_CorrectSize(self, w, h)
	 SCMainFrame_UpdateFramesSize(self, w, h)
	 SCMainFrame_UpdateFramesPositions(self, 1)
end
function SCMainFrame_OnMouseUp(self, button)
	if button == "LeftButton" then
		self:StopMovingOrSizing()
		--self:SetResizable(false)
		--self:SetScript("OnSizeChanged", nil)
	end
end

function SCMainFrame_OnMouseDown(self, button)
	if button == "LeftButton" and self:IsMovable() then
		self:StartMoving()
	elseif button == "RightButton" then
		--addon.sv.global.Scale = addon.sv.global.Scale + 0.1
		--addon.HandleOptions("Size")
		InterfaceOptionsFrame_OpenToCategory(addon.panel)
		InterfaceOptionsFrameOkay:Click("button", down)
		InterfaceOptionsFrame_OpenToCategory(addon.panel)
	end
end

function SCSpellFrame_OnUpdate(self, elapsed)
	SCSpellFrame_SetTimer(self, addon.sv.global.Timer)
	if GetTime()-self.start > self.dur then  -- For those cases when hide delay isn't needed
		if addon.sv.global.Timer then self.timer:SetText("") end
		if self.ch then
			self.ch = self.ch + 1
			self.charges:SetText(self.ch)
			if self.ch ~= addon.chargedSpells[self.spell] then
				self.start = GetTime()
				self.cd:SetCooldown(self.start, self.dur, self.ch, addon.chargedSpells[self.spell])
				if not addon.sv.global.Cooldown then self.cd:Hide() end
				self:SetScript("OnUpdate", SCSpellFrame_OnUpdate)
				return
			else
				SCMainFrame.chargedSpells[self.unitGUID][self.spell] = nil
				self.ch = nil
			end
		end
	end
	if GetTime()-self.start-1 > self.dur then   -- Hide with 1 sec delay 
		self.charges:SetText("")
		self.texture:SetTexture(0,0,1,0.6)
		self.spell = nil
		if SCMainFrame.wotf[self.unitGUID] and self.dur == 120 then
			SCMainFrame.wotf[self.unitGUID] = nil
		end
		self.unitGUID = nil
		self.dur = nil
		SCMainFrame_UpdateFramesPositions(SCMainFrame, self:GetID())
		if not addon.sv.global.Movable then self:Hide() end
		self:SetScript("OnUpdate", nil)
	end
end

function SCMainFrame_OnEvent(self, event, ...)
	local unit = select(1, ...)
	local _, unitRace = UnitRace(unit)
	local unitGUID = UnitGUID(unit)
	local spell = select(5, ...)
	local spellName = select(2, ...)
	if addon.spells[spell] and addon.units[unit] and ( UnitIsEnemy("player", unit) and not addon.sv.global.Friendly or 
		not UnitIsEnemy("player", unit) and addon.sv.global.Friendly and addon.sv.global.onlyFriendly or
		addon.sv.global.Friendly and not addon.sv.global.onlyFriendly ) then
		if self.currentID == SC_MAX_SPELLS + 1 then 
			print(SC_PREMSG..L["Too many cd's"]..". "..L["Spell"].." |c00AAFF00"..spellName.." |r "..L["was ignored"]..".")
			return 
		end
		if not FilterEvent(addon.events, spell, unit, select(6,...)) then return end
		local _, _, icon = GetSpellInfo(spell)
		local spellFrame = self[self.currentID]
		--print(self[self.currentID]:GetID())
		if not addon.chargedSpells[spell] then
			spellFrame.texture:SetTexture(icon)
			spellFrame:Show()
			self.currentID = self.currentID + 1
			spellFrame.start = GetTime()
		else
			if not self.chargedSpells[unitGUID] then 
				self.chargedSpells[unitGUID] = {}
			end
			if self.chargedSpells[unitGUID][spell] then 
				spellFrame = self[self.chargedSpells[unitGUID][spell]]
				spellFrame.ch = spellFrame.ch - 1
			else
				spellFrame.ch = addon.chargedSpells[spell] - 1
				self.chargedSpells[unitGUID][spell] = self.currentID
				self.currentID = self.currentID + 1
				spellFrame.texture:SetTexture(icon)
				spellFrame:Show()
				spellFrame.start = GetTime()
			end
			spellFrame.charges:SetText(spellFrame.ch)
		end
		if select(6, ...) then
			spellFrame.dur = select(6, ...)
		else
			spellFrame.dur = addon.spells[spell]
		end
		spellFrame.cd:SetCooldown(spellFrame.start, spellFrame.dur, spellFrame.ch, addon.chargedSpells[spell])
		if not addon.sv.global.Cooldown then spellFrame.cd:Hide() end
		spellFrame.spell = spell
		spellFrame.unitGUID = unitGUID
		spellFrame:SetScript("OnUpdate", SCSpellFrame_OnUpdate)
		if spell == 42292 and not self.wotf[unitGUID] and unitRace == "Scourge" then 
			self.wotf[unitGUID] = true
			SCMainFrame_OnEvent(self, event, unit, _, _, _, 7744, 30)
		elseif spell == 7744 and not self.wotf[unitGUID] then
			self.wotf[unitGUID] = true
			SCMainFrame_OnEvent(self, event, unit, _, _, _, 42292, 30)
		end
		if addon.preps[spell] then 
			SCMainFrame_OnPrepUsed(self, unitGUID)
		end
		--if spell == 116849 then --or spell == 1953 then --and unitGUID = UnitGUID("target")
			--PlaySoundFile("Interface\\Addons\\SomeCooldowns\\123.mp3", "Master")
		--end
	end
end

function SCSpellFrame_OnEnter(self)
	if not self.spell then return end
	GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
	GameTooltip:SetSpellByID(self.spell)
end

function SCSpellFrame_OnLeave(self)
	if not self.spell then return end
	GameTooltip:Hide()
end

function SlashCmdList.SOMECOOLDOWNS(msg)
	if msg == "reset" then
		print(SC_PREMSG, L["Cooldowns have been reset"]) 
		SCMainFrame_ResetCooldowns(SCMainFrame)
	else
		InterfaceOptionsFrame_OpenToCategory(addon.panel)
		InterfaceOptionsFrameOkay:Click("button", down)
		InterfaceOptionsFrame_OpenToCategory(addon.panel)
	end
end

function SC_RestoreDefaults()
	if SCOptions_Global then tremove(SCOptions_Global) end
	SCOptions_Global = addon.sv.defaults_global
end

function SC_SaveLoadOptions(self, event)
	if event == "ADDON_LOADED" then
		print(SC_PREMSG.." SomeCooldowns "..L["loaded"].."! "..L["Current version"]..": |cFF00CC00"..addon.version.."|r. "..L["Type"].." |cFF00CC00/sc|r "..L["to open a configuration window"])
		for i=1,SC_MAX_SPELLS do
			SCMainFrame[i] = _G["SCSpellFrame"..i]
			SCMainFrame[i].cd:SetSwipeColor(0, 0, 0, 0.8)
			SCMainFrame[i].cd:SetDrawEdge(true)
		end
		if not SCOptions_Global then
			SC_RestoreDefaults()
		end
		--print(SCOptions_Global.Pos.X)
		addon.sv.global = SCOptions_Global
		addon:UpdateVariables()
		SCMainFrame:SetPoint("BOTTOMLEFT", addon.sv.global.Pos.X, addon.sv.global.Pos.Y)
		if not addon.sv.global.Cooldown then
			addon.sv.global.Timer = true
		end
		addon.CreateConfig()
		addon.panel.checkbox1:SetChecked(addon.sv.global.Movable)
		addon.panel.checkbox2:SetChecked(addon.sv.global.Friendly)
		addon.panel.checkbox3:SetChecked(addon.sv.global.onlyFriendly)
		addon.panel.checkbox4:SetChecked(addon.sv.global.Reset)
		addon.panel.checkbox5:SetChecked(addon.sv.global.Cooldown)
		addon.panel.checkbox6:SetChecked(addon.sv.global.Timer)
		if not addon.sv.global.Friendly then
			addon.panel.checkbox3:Disable()
		else
			addon.panel.checkbox3:Enable()
		end
		if not addon.sv.global.Cooldown then
			addon.panel.checkbox6:Disable()
		else
			addon.panel.checkbox6:Enable()
		end
		addon.panel.slider:SetValue(addon.sv.global.Scale)
		addon.HandleOptions()
		self:UnregisterEvent("ADDON_LOADED")
	elseif event == "PLAYER_LOGOUT" then
		addon.sv.global.Pos.X = SCMainFrame:GetLeft()
		addon.sv.global.Pos.Y = SCMainFrame:GetBottom()
		if addon.panel.checkbox6:GetChecked() then
			addon.sv.global.Timer = true
		else
			addon.sv.global.Timer = false
		end
		SCOptions_Global = addon.sv.global
		--SC_RestoreDefaults()
	elseif event == "PLAYER_ENTERING_WORLD" then
		local _, instanceType = IsInInstance()
		if instanceType == "arena" or instanceType == "pvp" then
			print("|cFF00CC00SC:|r ", L["Cooldowns have been reset"]) 
			SCMainFrame_ResetCooldowns(SCMainFrame)
		end
	end
end
SLASH_SOMECOOLDOWNS1 = "/sc"

function addon.HandleOptions(msg)
	f = SCMainFrame
	global = addon.sv.global
	--char = addon.db.char[i]
	--f:SetPoint("BOTTOMLEFT",op.Pos.X, op.Pos.Y)
	if msg == "Size" or not msg then
		f:SetScale(global.Scale)
		-- f:SetSize(global.Size.Width*global.Scale, global.Size.Height*global.Scale)
		-- print(global.Scale)
		-- SCMainFrame_CorrectSize(f)
		-- SCMainFrame_UpdateFramesSize(f)
		-- SCMainFrame_UpdateFramesPositions(f)
	end
	if msg == "Movable" or not msg then
		SCMainFrame_SetToMove(f, global.Movable)
	end
	if msg == "Reset" or not msg then
		if global.Reset then
			addon.events:RegisterEvent("PLAYER_ENTERING_WORLD")
		else
			addon.events:UnregisterEvent("PLAYER_ENTERING_WORLD")
		end
	end
	if msg == "Timer" or not msg then
		if global.Timer then
			for i=1, SC_MAX_SPELLS do
				f[i].timer:Show()
			end
		else
			for i=1, SC_MAX_SPELLS do
				f[i].timer:Hide()
			end
		end
	end
	if msg == "Cooldown" or not msg then
		if global.Cooldown then
			for i=1, SC_MAX_SPELLS do
				f[i].cd:Show()
			end
		else
			for i=1, SC_MAX_SPELLS do
				f[i].cd:Hide()
			end
		end
	end
end

addon.events:RegisterEvent("ADDON_LOADED")
addon.events:RegisterEvent("PLAYER_LOGOUT")
addon.events:SetScript("OnEvent", SC_SaveLoadOptions)